/*
Copyright Danal Technology Inc. 2005
For instructional purposes only, not 
meant for production use.
*/
using System;
using Microsoft.SqlServer.Server;
using System.Data.SqlTypes;
using Microsoft.SqlServer;
using System.Reflection;

[Serializable]
[SqlUserDefinedType(Format.UserDefined, 
    MaxByteSize = 9,
    IsFixedLength =true,
    IsByteOrdered =true)]
public struct TimeDuration : INullable, IBinarySerialize 
{
    Boolean notNull;
    TimeSpan timeSpan;
    public override string ToString()
    {
        string s = "null";
        if (notNull)
        {
            s = timeSpan.ToString();
        }
        return s;
    }

    public bool IsNull
    {
        get
        {
            return !notNull;
        }
    }

    public static TimeDuration Null
    {
        get
        {
            TimeDuration h = new TimeDuration();
            return h;
        }
    }    
    public static TimeDuration Parse(SqlString s)
    {
        if (s.IsNull || s.Value.ToLower().Equals("null"))
            return Null;
        TimeDuration u = new TimeDuration();
        u.notNull = true;
        if(!TimeSpan.TryParse(s.Value, out u.timeSpan))
        {
            throw new ApplicationException("Impropertly formated TimeSpan");
        }
        return u;
    }

    static public implicit operator TimeDuration (TimeSpan timeSpan)
    {
        TimeDuration timeDuration = new TimeDuration();
        timeDuration.timeSpan = timeSpan;
        timeDuration.notNull = true;
        return timeDuration;
    }
    static public implicit operator TimeSpan(TimeDuration timeDuration)
    {
        return timeDuration.timeSpan;
    }


    public TimeSpan TimeSpan
    {
        get { return timeSpan; }
        set
        {
            notNull = true;
            timeSpan = value;
        }
    }

    #region IBinarySerialize Members

    public void Read(System.IO.BinaryReader r)
    {
        Byte b = r.ReadByte();
        notNull = b == 0 ? false : true;
        if (notNull)
        {
            Int64 ticks = 0L;
            UInt64 uticks = 0L;
            for (int index = 7; index >= 0; index--)
            {
                UInt64 t = r.ReadByte();
                uticks += (t << (index * 8));
            }
            if (uticks >= (UInt64)(Int64.MaxValue))
            {
                ticks = (Int64)(uticks - Int64.MaxValue);
            }
            else
            {
                ticks = (Int64)uticks;
                ticks += Int64.MinValue;
            }
            timeSpan = new TimeSpan(ticks);
        }
    }

    public void Write(System.IO.BinaryWriter w)
    {
        if (notNull)
        {
            w.Write((byte)1);
            Int64 ticks = timeSpan.Ticks;
            UInt64 uticks = 0L;
            if (ticks >= 0L)
            {

                uticks = (UInt64)ticks;
                uticks += (UInt64)(Int64.MaxValue);
            }
            else
            {
                uticks = (UInt64)(ticks - Int64.MinValue) ;

            }
            for (int index = 7; index >= 0; index--)
            {
                w.Write((byte)((uticks >> (index * 8)) & 0xFF));
            }
        }
        else
        {
            for (int index = 0; index < 9; index++)
            {
                w.Write((byte)0);
            }
        }
    }

    #endregion
}

