use Northwind
go

SELECT	RANK() OVER(ORDER BY City) as RANK,
		RANK() OVER(PARTITION BY City ORDER BY LastName) as PART_RANK,
		DENSE_RANK() OVER(ORDER BY City) as DENSE_RANK, 
		ROW_NUMBER() OVER(ORDER BY City) as ROW_NUM,
		NTILE(4) OVER(ORDER BY City) as NTILE_4,
		LastName, 
		FirstName, 
		City 
FROM	Employees 
ORDER BY City, LastName

-- paging
WITH OrderedOrders AS 
(select SalesOrderID, OrderDate,
Row_Number() OVER (order by OrderDate)as RN  
from SalesOrderHeader ) 
SELECT * from OrderedOrders WHERE RN between 50 and 60

-- pivot
use tempdb
go

create table CarSales (Make varchar(6), Year int, Sales int)
go
insert into CarSales values ('Honda', 1990, 2000)
insert into CarSales values ('Honda', 1990, 1000)
insert into CarSales values ('Acura', 1990, 500)
insert into CarSales values ('Honda', 1991, 3000)
insert into CarSales values ('Acura', 1991, 300)
insert into CarSales values ('Acura', 1991, 600)
insert into CarSales values ('Acura', 1992, 800)
go

select 	* 
into 	CarSalesPivot
from 	CarSales PIVOT(SUM(Sales) FOR Year IN ([1990], [1991])) t
go

drop table CarSales
drop table CarSalesPivot
 
-- The PIVOT is performed as follows
--  All columns of the table are broken into 
-- The PIVOT column; selected values of this column will become columns of the pivot table (this is the Year in the example above
-- The column with aggregation function defined. This is know also as the value column. This is determined by SUM(Sales) in the example above.
-- Rest of the columns become become grouping columns. This is the Make column in our example.
-- 2. Set of PIVOT column constants determines the columns in the PIVOT output table. This is 1990 and 1991 in our example.
-- 3. The grouping columns determine the rows in the PIVOT output table. There is one row for each distinct combination of values of grouping columns.
-- The following two steps then derive the values in the output table:
-- 4. Rows that do not match any PIVOT constant in the list are removed
-- 5. Rows that have the same value in the rest of the columns are aggregated using the agg function inside the PIVOT. 
-- 
-- Some notes:
-- The data type of the pivot_column  should support equality comparisons and support an implicit or explicit conversion from nvarchar. This excludes text and image data types. 
-- A table_alias is required at the end which becomes the associated table-name for all columns output by the PIVOT operator including the grouping columns. It is t on the slide here.

-- note: use the CarSalesPivot table created in the PIVOT example
select 	* 
from 	CarSalesPivot UNPIVOT
	(Sales for  Year in ([1990], [1991])) t

-- Notes:
-- 1. Note that PIVOT followed by an UNPIVOT on the same columns does not necessarily produce the same result. Rows can be lost because of two reasons: row does not contain one of the listed values in the PIVOT column; rows that have equal values for all grouping columns are aggregated and the split information is lost.  

-- TOP (@P)
declare @r int
select  @r = 10
select top(@r) * from pubs.dbo.sales
go

-- CTE

-- select top 3 departments with most females
use AdventureWorks
go

select	top(3) with ties D.name,cnt [Females]
from	(select count(*) cnt, DepartmentID dept from Employee
			where 	Gender = 'F' 
			group by DepartmentID) as F, Department D 
where F.dept = D.DepartmentID
order by cnt desc

WITH Fcount (cnt, dept) as 
(select count(*),DepartmentID from Employee
where Gender='F' group by DepartmentID) 
SELECT top (3)with ties D.name,cnt [Females]
from Fcount F join Department D 
ON F.dept=D.DepartmentID
order by cnt desc

-- recursion
Use AdventureWorks
go 
WITH EmpCTE(empid,  mgrid)
AS
( 
  SELECT EmployeeID,  ManagerID  
  FROM Employee
  WHERE EmployeeID = 1
  		UNION ALL
  SELECT E.EmployeeID, E.ManagerID
  FROM Employee AS E JOIN EmpCTE AS M
      ON E.ManagerID = M.empid
)
SELECT * FROM EmpCTE

--It is possible to restrict internally the number of levels as follows:

WITH EmpCTE(empid,  mgrid, level)
AS
( 
Use AdventureWorks
go  
SELECT EmployeeID,  ManagerID, 0  
  FROM Employee
  WHERE EmployeeID = 1 -- anchor
  		UNION ALL
  SELECT E.EmployeeID, E.ManagerID, level+1
  FROM Employee AS E JOIN EmpCTE AS M
      ON E.ManagerID = M.empid
	 WHERE level<2
)
SELECT * FROM EmpCTE
