
use pubs
go

SELECT a.au_id AS [AuthorID]
     , a.au_fname + ' ' + a.au_lname AS [AuthorName]
     , count(*) AS [NumBooks]
     , avg(royaltyper) AS RoyaltyPercent
     , (SELECT avg(royaltyper) FROM titleauthor) AS [AverageRoyalty]
FROM authors a
JOIN titleauthor ta
  ON a.au_id = ta.au_id
WHERE a.state = 'CA'
GROUP BY ALL a.au_id, a.au_lname, a.au_fname
HAVING avg(royaltyper) > (SELECT avg(royaltyper) FROM titleauthor)
ORDER BY avg(royaltyper) DESC;


-- PIVOT UNPIVOT
-- CROSS APPLY
-- OUTER APPLY

SELECT a.au_id AS [AuthorID]
     , a.au_fname + ' ' + a.au_lname AS [AuthorName]
     , count(*) AS [NumBooks]
     , avg(royaltyper) AS [RoyaltyPercent]
     , av.AvgRoy AS [AverageRoyalty]
FROM authors a
JOIN titleauthor ta
  ON a.au_id = ta.au_id
CROSS JOIN (SELECT avg(royaltyper) as AvgRoy FROM titleauthor) AS av
WHERE a.state = 'CA'
GROUP BY a.au_id, a.au_lname, a.au_fname, av.AvgRoy
HAVING avg(royaltyper) > avg(av.AvgRoy)
ORDER BY avg(royaltyper) desc;


-- CTE version
;WITH AR (AvgRoy) AS
(
 select avg(royaltyper) from titleauthor
)
SELECT a.au_id AS [AuthorID]
     , a.au_fname + ' ' + a.au_lname AS [AuthorName]
     , count(*) AS [NumBooks]
     , avg(royaltyper) AS [RoyaltyPercent]
     , AR.AvgRoy AS [AverageRoyalty]
FROM authors a
JOIN titleauthor ta
  ON a.au_id = ta.au_id
CROSS JOIN AR
WHERE a.state = 'CA'
GROUP BY a.au_id, a.au_lname, a.au_fname, AR.AvgRoy
HAVING avg(royaltyper) > AR.AvgRoy
ORDER BY avg(royaltyper) desc;



