
USE SodaDB
GO

-- 
-- Step 1:
-- CREATE the MESSAGE TYPES and CONTRACT for this program
-- CREATE the QUEUES and SERVICES for this program
--

IF EXISTS (SELECT * FROM sys.services WHERE name = 'InvService_L5A')
    DROP SERVICE InvService_L5A;
GO
IF EXISTS 
  (SELECT * FROM sys.service_queues WHERE name = 'InvRequestQueue_L5A')
    DROP QUEUE InvRequestQueue_L5A;
GO

IF EXISTS (SELECT * FROM sys.services WHERE name = 'InvInitiator_L5A')
    DROP SERVICE InvInitiator_L5A;
GO

IF EXISTS 
  (SELECT * FROM sys.service_queues WHERE name = 'InvResponseQueue_L5A')
    DROP QUEUE InvResponseQueue_L5A;
GO

IF EXISTS 
  (SELECT * FROM sys.service_contracts WHERE name = 'InvContract_L5A')
    DROP CONTRACT InvContract_L5A;
GO

IF EXISTS 
  (SELECT * FROM sys.service_message_types WHERE name = 'InvRequestMsg_L5A')
    DROP MESSAGE TYPE InvRequestMsg_L5A;
GO

IF EXISTS 
  (SELECT * FROM sys.service_message_types WHERE name = 'InvResponseMsg_L5A')
    DROP MESSAGE TYPE InvResponseMsg_L5A;
GO

-- define two message types with XML validation
CREATE MESSAGE TYPE InvRequestMsg_L5A 
VALIDATION = WELL_FORMED_XML
GO

CREATE MESSAGE TYPE InvResponseMsg_L5A
VALIDATION = WELL_FORMED_XML
GO

-- define a contract between the two sides
CREATE CONTRACT InvContract_L5A
 (InvRequestMsg_L5A SENT BY INITIATOR,
  InvResponseMsg_L5A SENT BY TARGET)
GO

-- create queues and services
CREATE QUEUE InvRequestQueue_L5A
GO

CREATE QUEUE InvResponseQueue_L5A
GO

CREATE SERVICE InvService_L5A
 ON QUEUE InvRequestQueue_L5A
 (InvContract_L5A)
GO

CREATE SERVICE InvInitiator_L5A
 ON QUEUE InvResponseQueue_L5A
 (InvContract_L5A)
GO



