--*********************************************
--*  Create a database
--*********************************************
use master;

if exists (select * from sys.databases where name = 'Inventory')
begin
	print 'dropping database ''Inventory''';
	drop database Inventory;
end

create database Inventory;
go

use Inventory;
go

--*********************************************
--*  Define the messages and contracts required
--* to talk to the [Inventory] service
--*********************************************
create message type
  [http://www.LotsOfBooks.com/msg/StockUpdate]
  validation = WELL_FORMED_XML;

create contract [http://www.LotsOfBooks.com/contract/UpdateInventory/v1.0]
( 
	[http://www.LotsOfBooks.com/msg/StockUpdate] sent by initiator
);

-- Next contract

create message type [http://www.LotsOfBooks.com/msg/InStockQuery]
    validation = WELL_FORMED_XML;

create message type [http://www.LotsOfBooks.com/msg/InStockResponse]
    validation = WELL_FORMED_XML;

create contract [http://www.LotsOfBooks.com/contract/CheckInventory/v1.0]
(
    [http://www.LotsOfBooks.com/msg/InStockQuery] SENT BY INITIATOR,
    [http://www.LotsOfBooks.com/msg/InStockResponse] SENT BY TARGET
);

--*********************************************
--*  Create the [Inventory] service
--*********************************************
create queue [Inventory Queue];
create service [Inventory] 
     on queue [Inventory Queue] 
(
	[http://www.LotsOfBooks.com/contract/UpdateInventory/v1.0],
	[http://www.LotsOfBooks.com/contract/CheckInventory/v1.0]
);


--*********************************************
--*  Create the [Inventory Client] service
--*********************************************
create queue [Inventory Client Queue];
create service [Inventory Client] 
     on queue [Inventory Client Queue];
 
--*********************************************
--*  Look at the objects we just created
--*********************************************

--select * from sys.service_queues where name = 'Inventory Queue';
--select * from sys.services where name = 'Inventory';

--select * from [Inventory Queue];
--receive * from [Inventory Queue];

--******************************************************
--*  Create a sample Inventory database and populate it
--******************************************************
if exists(select * from sysobjects where name=N'Inventory' and type='U')
	drop table Inventory;

create table Inventory (
		TitleId		nvarchar(6) not null,
		Quantity	int not null, 
		primary key (TitleId, Quantity));

insert Inventory values ('PS1372', 200);
insert Inventory values ('PC1035', 200);
insert Inventory values ('BU1111', 200);
insert Inventory values ('PS7777', 200);
insert Inventory values ('TC4203', 200);
insert Inventory values ('PS2091', 200);
insert Inventory values ('PS2106', 200);
insert Inventory values ('PC9999', 200);
insert Inventory values ('TC3218', 200);
insert Inventory values ('PS3333', 200);
insert Inventory values ('PC8888', 200);
insert Inventory values ('MC2222', 200);
insert Inventory values ('BU7832', 200);
insert Inventory values ('TC7777', 200);
insert Inventory values ('BU1032', 200);
insert Inventory values ('MC3021', 200);
insert Inventory values ('MC3026', 200);
insert Inventory values ('BU2075', 200);

--select * from Inventory

use master;