using System;
using System.Threading;
using System.Data;
using System.Xml;
using System.IO;
using System.Data.SqlClient;
using System.Data.SqlTypes;

namespace InventoryNamespace
{
	class Inventory
	{
		Broker broker = new Broker();

		#region Constant Declarations

		// incoming Purchase Request (PO)
		//
		//static readonly string PO_XML_NS = "http://orders.lotsofbooks.com/Operations/BookPurchase";
		//static readonly string PO_XML_NS_PREFIX = "po";
		static readonly string PO_XML_TITLEID = "TitleId";
		static readonly string PO_XML_TITLE_QUANTITY = "Quantity";

		// Inventory Response
		//
		static readonly string PO_XML_ROOT2 = "InventoryResponse";
		static readonly string PO_XML_RESPONSE = "Response";

		//Delegate to update the UI
		public delegate void pLogInfo(string msg);
		private pLogInfo m_log=null;

		#endregion

		#region Main Loop - ProcessOrders

		public void ProcessOrders (AutoResetEvent shutdown, pLogInfo log)
		{
			// Receive an XML message
			//
			while (true)
			{
				String		msg;
				String		msgType;
				Guid		dialogHandle;
				Guid		serviceInstance;

				//Check if asked to break the loop
				if (shutdown.WaitOne(1,false))
					break;

				//assign delegate for UI
				m_log=log;

				broker.tran = broker.conn.BeginTransaction();

				broker.Receive (out msgType, out msg, out serviceInstance, out dialogHandle);
				if (shutdown.WaitOne(1, false))
					break;

				if (msg == null)
				{
					broker.tran.Commit();
					break;
				}

				switch (msgType)
				{			
					case "http://www.LotsOfBooks.com/msg/InStockQuery":
						ProcessInventoryRequest (msg, serviceInstance, dialogHandle);
						break;

					case "http://www.LotsOfBooks.com/msg/StockUpdate":
						ProcessStockUpdate (msg, serviceInstance, dialogHandle);
						break;

					case "http://schemas.microsoft.com/SQL/ServiceBroker/EndDialog":
						broker.EndDialog (dialogHandle);
						break;

					case "http://schemas.microsoft.com/SQL/ServiceBroker/Error":
						broker.EndDialog (dialogHandle);
						break;
				}
				broker.tran.Commit();
			}
		}

		#endregion

		#region Process Inventory Request

		private void ProcessInventoryRequest (String msg, Guid serviceInstance, Guid dialogHandle)
		{
			Console.WriteLine("msg is " + msg + "dh is " + dialogHandle.ToString());

			// Load XML msg into doc
			//
			XmlDocument doc = new XmlDocument();
			doc.LoadXml(msg);

			// Locate elements in DOM
			XmlNodeList list = doc.GetElementsByTagName(PO_XML_TITLEID);
			String TitleId = list.Item(0).InnerXml;

			list = doc.GetElementsByTagName(PO_XML_TITLE_QUANTITY);
			int Quantity = Convert.ToInt32(list.Item(0).InnerXml);
			
			// Check Inventory
			//
			m_log("Inventory Check... " + TitleId + ": " + Quantity.ToString());

			bool rc = CheckInventory (TitleId, Quantity);

			
				
			SendCustomerReply (dialogHandle, rc);

			return;	
		}

		#endregion

		#region Process Stock Update

		private void ProcessStockUpdate (String msg, Guid serviceInstance, Guid dialogHandle)
		{
			Console.WriteLine("msg is " + msg + "dh is " + dialogHandle.ToString());

			// Load XML msg into doc
			//
			XmlDocument doc = new XmlDocument();
			doc.LoadXml(msg);

			// Locate elements in DOM
			XmlNodeList list = doc.GetElementsByTagName(PO_XML_TITLEID);
			String TitleId = list.Item(0).InnerXml;

			list = doc.GetElementsByTagName(PO_XML_TITLE_QUANTITY);
			int Quantity = Convert.ToInt32(list.Item(0).InnerXml);
			
			// Check Inventory
			//
			AddToInventory (TitleId, Quantity);
			
			broker.EndDialog (dialogHandle);

			return;	
		}

		#endregion

		#region Add To Inventory

		private void AddToInventory (String TitleId, int Quantity)
		{
			// Create Instance of Connection and Command Object
			SqlConnection myConnection = broker.conn;

			SqlCommand myCommand = new SqlCommand("Update Inventory Set Quantity=Quantity + @Quantity  " +
				"WHERE TitleId = @TitleId ", myConnection);
			myCommand.Transaction = broker.tran;

			// Add Parameters 
			SqlParameter paramTitleId = new SqlParameter("@TitleId", SqlDbType.NVarChar, 6);
			paramTitleId.Value = TitleId;
			myCommand.Parameters.Add(paramTitleId);

			SqlParameter paramQuantity = new SqlParameter("@Quantity", SqlDbType.Int);
			paramQuantity.Value = Quantity;
			myCommand.Parameters.Add(paramQuantity);

			try
			{
				myCommand.ExecuteNonQuery();
				m_log("Increased Inventory... " + TitleId + ": " + Quantity.ToString());
			}
			catch (Exception e)
			{
				Console.WriteLine("Update of inventory failed " + e.Message);
			}
		}

		#endregion

		#region Subtract From Inventory

		private void SubtractFromInventory (String TitleId, int Quantity)
		{
			// Create Instance of Connection and Command Object
			SqlConnection myConnection = broker.conn;

			SqlCommand myCommand = new SqlCommand("Update Inventory Set Quantity=Quantity - " + Quantity.ToString() + 
				" WHERE TitleId = '" + TitleId.ToString() + "'", myConnection);
			myCommand.Transaction = broker.tran;

/*
			// Add Parameters 
			SqlParameter paramTitleId = new SqlParameter("@TitleId", SqlDbType.NVarChar, 6);
			paramTitleId.Value = TitleId;
			myCommand.Parameters.Add(paramTitleId);

			SqlParameter paramQuantity = new SqlParameter("@Quantity", SqlDbType.Int);
			paramQuantity.Value = Quantity;
			myCommand.Parameters.Add(paramQuantity);
*/
			try
			{
				myCommand.ExecuteNonQuery();
				
				m_log("Reduced Inventory... " + TitleId + ": " + Quantity.ToString());
			}
			catch (Exception e)
			{
				Console.WriteLine("Update of inventory failed " + e.Message);
			}
		}

		#endregion

		#region Check Inventory

		private bool CheckInventory (String TitleId, int Quantity)
		{
			int RealQuantity;

			// Create Instance of Connection and Command Object
			SqlConnection myConnection = broker.conn;
			SqlCommand myCommand = new SqlCommand("SELECT Quantity FROM InventoryDB.dbo.Inventory " +
												  "WHERE TitleId = @TitleId ", myConnection);
			myCommand.Transaction = broker.tran;

			// Add Parameters 
			SqlParameter paramTitleId = new SqlParameter("@TitleId", SqlDbType.NVarChar, 6);
			paramTitleId.Value = TitleId;
			myCommand.Parameters.Add(paramTitleId);

			try
			{
				SqlDataReader result = myCommand.ExecuteReader();
				if (result.Read())
				{
					RealQuantity = result.GetInt32(0);

					result.Close();

					if (Quantity <= RealQuantity)
					{
						SubtractFromInventory(TitleId, Quantity);
						return (true);
					}
					else
					{
						return (false);
					}
				}
				else
				{
					result.Close();
					return (false);
				}
			}
			catch (Exception e)
			{
				Console.WriteLine("Select from state table failed " + e.Message);
				return (false);
			}
		}

		#endregion

		#region Send Customer Reply

		private void SendCustomerReply (Guid dialogHandle, bool InStockFlag)
		{
			// build XML reply
			//
			XmlDocument doc = new XmlDocument();
			//XmlElement root = doc.CreateElement(PO_XML_NS_PREFIX ,PO_XML_ROOT2,PO_XML_NS);
            XmlElement root = doc.CreateElement(PO_XML_ROOT2);
            doc.AppendChild(root);
			
			XmlElement Response;
			Response = doc.CreateElement(PO_XML_RESPONSE);
			if (InStockFlag == true)
			{
				Response.InnerText="In stock";
			}
			else
			{
				Response.InnerText="Out of stock";
			}
			root.AppendChild(Response);

			// Send the msg
			//
			broker.Send (dialogHandle, doc.InnerXml);

			// End the dialog
			//
			broker.EndDialog (dialogHandle);

			return;
		}

		#endregion	

		#region Shutdown
		public void Shutdown()
		{
			broker.cmd.Cancel();
		}
		#endregion
	}

	
}