using System;
using System.Configuration;
using System.Data;
using System.Data.SqlClient;
using System.Data.SqlTypes;

	/// <summary>
	/// Summary description for Broker.
	/// </summary>
	public class Broker
	{
		public SqlConnection conn;
		public SqlTransaction tran;
		public SqlCommand cmd;

		#region Send

		public void Send(Guid dialogHandle, String msg)
		{
			// Get the context command
			cmd = conn.CreateCommand();
			cmd.Transaction = tran;

			// Add dialog handle
			SqlParameter paramDialogHandle = new SqlParameter("@dh", SqlDbType.UniqueIdentifier);
			paramDialogHandle.Value = dialogHandle;
			cmd.Parameters.Add(paramDialogHandle);	
		
			// Add message
			SqlParameter paramMsg = new SqlParameter("@msg", SqlDbType.NVarChar, msg.Length);
			paramMsg.Value = msg;
			cmd.Parameters.Add(paramMsg);	
			
			// Build the SEND command
			cmd.CommandText="SEND ON CONVERSATION @dh " +
				"MESSAGE TYPE [http://www.LotsOfBooks.com/msg/InStockResponse] " +
				"(@msg)";

			try
			{
				int rc = cmd.ExecuteNonQuery();

				//Console.WriteLine("rc is {0}", rc);
			}
			catch (Exception e)
			{
				//Console.WriteLine("SEND failed " + e.Message);
			}
			return;
		}


		#endregion

		#region Receive

		public void Receive(out String msgType, out String msg, out Guid ConversationGroup, out Guid dialogHandle)
		{
			//default return values
			msgType = null;
			msg = null;
			ConversationGroup = Guid.Empty;
			dialogHandle = Guid.Empty;

			// Get the context command
			cmd=conn.CreateCommand();
			cmd.Transaction = tran;

			// Get output msgtype
			SqlParameter paramMsgType = new SqlParameter("@msgtype", SqlDbType.NVarChar, 256);
			paramMsgType.Direction = ParameterDirection.Output;
			cmd.Parameters.Add(paramMsgType);

			// Get output msg
			SqlParameter paramMsg = new SqlParameter("@msg", SqlDbType.NVarChar, 4000);
			paramMsg.Direction = ParameterDirection.Output;
			cmd.Parameters.Add(paramMsg);

			// Get output si
			SqlParameter paramConversationGroup = new SqlParameter("@cg", SqlDbType.UniqueIdentifier);
			paramConversationGroup.Direction = ParameterDirection.Output;
			cmd.Parameters.Add(paramConversationGroup);

			// Get output dh
			SqlParameter paramDialogHandle = new SqlParameter("@dh", SqlDbType.UniqueIdentifier);
			paramDialogHandle.Direction = ParameterDirection.Output;
			cmd.Parameters.Add(paramDialogHandle);

			// Build the Receive command
			cmd.CommandText="WAITFOR (RECEIVE TOP(1)  @msgtype=message_type_name, " +
				"@msg=message_body, " +
				"@cg=conversation_group_id, " +
				"@dh=conversation_handle " +
				"FROM [Inventory Queue]) " +
				", TIMEOUT 5000";

			try
			{
				int rc = cmd.ExecuteNonQuery();

				if (!(paramMsgType.Value is DBNull))
				{
					msgType = (String)paramMsgType.Value;
					msg = (String)paramMsg.Value;
					ConversationGroup = (System.Guid)paramConversationGroup.Value;
					dialogHandle = (System.Guid)paramDialogHandle.Value;
				}

				//Console.WriteLine("rc is {0}", rc);
			}
			catch (Exception e)
			{
				//Console.WriteLine("RECEIVE failed " + e.Message);
			}

			return;
		}


		#endregion

		#region End Dialog

		public void EndDialog(Guid dialogHandle)
		{
			// Get the context command
			SqlCommand cmd=conn.CreateCommand();
			cmd.Transaction = tran;

			// Add dialog handle
			SqlParameter paramDialogHandle = new SqlParameter("@dh", SqlDbType.UniqueIdentifier);
			paramDialogHandle.Value = dialogHandle;
			cmd.Parameters.Add(paramDialogHandle);	
		
			// Build the SEND command
			cmd.CommandText="END CONVERSATION @dh ";

			try
			{
				int rc = cmd.ExecuteNonQuery();

				//Console.WriteLine("rc is {0}", rc);
			}
			catch (Exception e)
			{
				//Console.WriteLine("END CONVERSATION failed " + e.Message);
			}
			return;
		}

		#endregion
	
	}


